package CVSMonitor::HTML::Form;

# A stripped HTML componant generation package

use strict;
use UNIVERSAL 'isa';





#####################################################################
# Basic Form Componant Generators

# Creates a drop box
sub dropbox {
	my $class = shift;
	my $name = shift or return undef;
	my $data = shift or return undef;
	my $value = shift;
	$value = '' unless defined $value;
	my $opt = shift || {};

	# Create the options array, and add all the data options
	my $havenull = 0;
	my @options = ();
	foreach ( @$data ) {
		$_->[0] = '' unless defined $_->[0];
		$havenull = 1 if $_->[0] eq '';

		# Escape the label if needed
		$_->[1] = CVSMonitor::HTML->escape($_->[1]) unless $opt->{dontescape};

		# Add the option to the array
		push @options, $value eq $_->[0]
			? "<option value=\"$_->[0]\" selected>$_->[1]</option>\n"
			: "<option value=\"$_->[0]\">$_->[1]</option>\n";
	}

	# Add the null entry if we need it
	unless ( $havenull or ($opt->{size} and $opt->{size} ne '1') or ($opt->{notnull} and $value ne '') ) {
		$opt->{nullstring} = '' unless defined $opt->{nullstring};
		unshift @options, $value ne ''
			? "<option value=\"\">$opt->{nullstring}</option>\n"
			: "<option value=\"\" selected>$opt->{nullstring}</option>\n";
	}

	# Create the tag properties
	my %props = ( name => $name );
	if ( ! defined $opt->{size} ) {
		# Do nothing
	} elsif ( $opt->{size} eq 'auto' ) {
		$props{size} = @options;
	} elsif ( $opt->{size} > 0 ) {
		$props{size} = $opt->{size};
	}
	foreach ( qw{id class onChange} ) {
		$props{$_} = $opt->{$_} if exists $opt->{$_};
	}

	CVSMonitor::HTML->tag_pair( 'select', \%props, join( '', @options ) );
}

# Generate a cluster of radio buttons
# Create a set of radio buttons
sub radio {
	my $class = shift;
	my $name = shift or return undef;
	my $data = shift or return '';
	my $value = shift;
	$value = '' unless defined $value;
	my $layout = shift or return undef;
	my $opts = shift || {};

	# Map the value half of the data into radio buttons
	foreach ( @$data ) {
		$_->[0] = '' unless defined $_->[0];

		# Create the radio button
		my $props = {
			type  => 'radio',
			name  => $name,
			value => $_->[0],
			};
		$props->{id} = "$name.$_->[0]" if $opts->{autoid} or $opts->{activelabels};
		$props->{checked} = undef if $_->[0] eq $value;
		$props->{value} = CVSMonitor::HTML->escape($props->{value}) unless $opts->{dontescape};
		$_->[0] = CVSMonitor::HTML->tag( 'input', $props );

		# If they want active labels, map the labels to label tags
		if ( $opts->{activelabels} ) {
			$_->[1] = CVSMonitor::HTML->label( $props->{id}, $_->[1] );
		}
	}

	# Now, hand them over to the appropriate layout method
	$layout ||= 'vertical';
	if ( $layout eq 'vertical' ) {
		return join "<br>\n", map { "$_->[0]$_->[1]" } @$data;

	} elsif ( $layout eq 'horizontal' ) {
		return join " \n", map { $_->[0] . $_->[1] } @$data;

	} elsif ( $layout eq 'table' ) {
		# Generate the cell pairs
		my @cellpairs = ();
		my %cellprops = ref $opts->{td} ? %{$opts->{td}} : ();
		$cellprops{valign} ||= 'middle';
		$cellprops{nowrap} = undef if $opts->{nowrap};

		foreach ( @$data ) {
			push @cellpairs, "    "
				. CVSMonitor::HTML->tag_pair( 'td', \%cellprops, $_->[0] )
				. "\n    "
				. CVSMonitor::HTML->tag_pair( 'td', \%cellprops, $_->[1] )
				. "\n";
		}

		# Format the cell pairs into a grid
		my @rows = ();
		my $cols = $opts->{cols} ? $opts->{cols}
			: $opts->{rows} ? ceil(scalar(@cellpairs) / $opts->{rows})
			: 1;
		while ( scalar @cellpairs ) {
			push @rows, join( '', splice(@cellpairs, 0, $cols) );
		}

		# Generate the final table html
		my $table = join( '', map { "\n  <tr>\n$_  </tr>" } @rows ) . "\n";
		my %tableprops = ref $opts->{table} ? %{$opts->{table}} : ();
		$tableprops{border} ||= 0;
		$tableprops{cellspacing} ||= 0;
		$tableprops{cellpadding} = 1 unless defined $tableprops{cellpadding};
		return CVSMonitor::HTML->tag_pair( 'table', \%tableprops, $table );

	} elsif ( isa( $layout, 'CODE' ) ) {
		# Hand off to the external layout method
		return &{$layout}($data, $opts);
	}

	"Unknown radio button layout format '$layout'";
}

# Generate a checkbox
sub checkbox {
	my $class = shift;
	my $name = shift or return undef;
	my $value = shift;
	$value = '' unless defined $value;
	my $opt = shift || {};
	my %props = ( type => 'checkbox', name => $name, value => $value );
	$value = CVSMonitor::HTML->escape( $value ) unless $opt->{dontescape};

	# Pass through options
	foreach ( qw{class id onClick} ) {
		$props{$_} = $opt->{$_} if defined $opt->{$_};
	}

	# Handle the checked flag
	if ( $opt->{checked} ) {
		$props{checked} = undef;
	} else {
		delete $props{checked};
	}

	# Handle the disabled option
	if ( $opt->{disabled} ) {
		$props{disabled} = 'true';
	} else {
		delete $props{disabled};
	}

	CVSMonitor::HTML->tag( 'input', \%props );
}






#####################################################################
# Complex Form Element Generators

# Given a Field object, and an optional namespace, create the root CGI namespace
sub cgi_name { (defined $_[2] and $_[2] ne '') ? "$_[2].$_[1]->{Name}" : $_[1]->{Name} }

# A "present" hidden tag is used when a control can be on a page in a
# submitted form, but return no data to the script
# It lets the script know the control is in the page
sub present { defined $_[1] and "<input type=\"hidden\" name=\"$_[1].Present\" value=\"1\">" }

# Recreate all the parameters in the form of hiddens.
# First argument can optionally be a CGI object.
# Any other arguments are a list of names of parameters NOT to include.
sub hidden_params {
	my $query = UNIVERSAL::can( $_[0], 'param' ) ? shift : CGI->new or return undef;
	my %skip = map { $_, 1 } @_;

	# Get the params and map them to hidden fields
	my @hidden = ();
	foreach my $param ( grep { ! $skip{$_} } $query->param ) {
		my $escaped_param = CVSMonitor::HTML->escape( $param );

		# Each parameter name can have multiple values
		push @hidden, map {
			CVSMonitor::HTML->tag( 'input', {
				type  => 'hidden',
				name  => $escaped_param,
				value => CVSMonitor::HTML->escape($_),
			} )
		} $query->param( $param );
	}

	# Return as a single string
	@hidden ? join( "\n", @hidden ) : '';
}

1;
