package CVSMonitor::Range::Basics;

# Provides an implementation of a set of basic numeric values.

use strict;
use UNIVERSAL qw{isa can};
use base 'CVSMonitor::Range';

use vars qw{@field_list};
BEGIN {
	@field_list = qw{add remove increase decrease karma};
}



#####################################################################
# Functional Methods

# Add Version information to the Range totals
sub add {
	my $self = shift;
	my $Source = shift or return undef;
	return undef unless can( $Source, 'getChangeSets' );
	my $ChangeSets = $Source->getChangeSets or return undef;

	# Index the dates
	my %totals = ();
	$totals{$_->{date}} = $_ foreach @$self;

	# Work out what level of depth to use
	my $regex = CVSMonitor::DateDepth->isolate_regex( $self->depth ) or return undef;

	# Iterate over the change sets
	foreach my $ChangeSet ( @$ChangeSets ) {
		return undef unless $ChangeSet->date =~ $regex;
		next unless exists $totals{$1};
		my $key = $1;

		# Add to the changes totals
		$totals{$key}->{commits}++;
		$totals{$key}->{changes} += $ChangeSet->changes;

		# Get the line information
		my $lines = $ChangeSet->getBalancedLines;
		foreach ( @field_list ) {
			next unless defined $totals{$key}->{$_};
			$totals{$key}->{$_} += $lines->{$_};
		}
	}

	1;
}

# Get a hash of the totals for the Range
sub totals {
	my $self = shift;

	my $totals = {
		commits  => 0,
		changes  => 0,

		add      => 0,
		remove   => 0,
		increase => 0,
		decrease => 0,

		karma    => 0,
		};

	# Add up the dates
	foreach my $date ( @$self ) {
		foreach ( keys %$totals ) {
			$totals->{$_} += $date->{$_};
		}
	}

	# Add the extra stuff
	$totals->{up}       = $totals->{add} + $totals->{increase};
	$totals->{down}     = $totals->{remove} + $totals->{decrease};
	$totals->{activity} = $totals->{up} + $totals->{down};
	$totals->{change}   = $totals->{up} - $totals->{down};

	$totals;
}

# For a given date, generate it's basic stats struct
sub getStatisticsStruct {
	return undef unless isa( $_[1], 'CVSMonitor::DateDepth' );
	return {
		date     => $_[1]->toString,
		commits  => 0,
		changes  => 0,
		add      => 0, 
		remove   => 0,
		increase => 0,
		decrease => 0,
		karma    => 0,
		};
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
