package CVSMonitor::Util;

# A package that provides a set of importable convenience functions.
# This is seperate the from the main code tree. Provides a similar set
# of functions to those in Scalar::Util and List::Util.

use strict;
use UNIVERSAL 'isa';
use base 'Exporter';
use File::Spec::Unix ();
use File::Spec ();

# If you really want to, you can import any of our stuff
our @EXPORT_OK = qw{any distinct string integer dir path};

# Are we currently using Unix filesystem semantics?
my $UNIX;
BEGIN { $UNIX = $File::Spec::ISA[0] eq 'File::Spec::Unix'; }





#####################################################################
# Utility Functions

# The 'any' logical list test
sub any(&@) { my $code = shift; foreach ( @_ ) { return 1 if &{$code}() } '' }

# Takes a list of anything
sub distinct(@) { my %seen = (); grep { ! $seen{$_}++ } @_ }

# An often needed but sometimes tricky test.
# "Is a value a defined string of at least one character"
sub string {
	defined $_[0] and ! ref $_[0] and !! length($_[0]);
}

# Tries to determine if the arg is a normal positive interger.
# Returns it if so, or undef otherwise
sub integer {
	my $integer = shift;
	return undef if ! defined $integer or ref $integer;
	$integer =~ /^[1-9]\d*$/ ? $integer : undef;
}

# For a given file, get the directory it is contained in.
# Do so in a way that works on all operating systems.
sub dir($) {
	my $filename = string($_[0]) ? shift : return undef;
	my ($volume, $dirs, $file) = File::Spec->splitpath( $filename );
	File::Spec->catpath( $volume, $dirs );
}

BEGIN {
eval ($UNIX ? <<'END_UNIX' : <<'END_NONUNIX');
sub path($) { shift }
END_UNIX
# Take a filesystem path in Unix format, and convert it to whatever
# we are currently on.
sub path($) {
	# Split the Unix path
	my ($volume, $dir, $file) = File::Spec::Unix->splitpath($_[0]);
	my @dirs = File::Spec::Unix->splitdir($dir);

	# Fix any special dirs
	@dirs = map {
		($_ eq '.')  ? File::Spec->curdir :
		($_ eq '..') ? File::Spec->updir :
		$_
		} @dirs;

	# Join everything back together
	$dir = File::Spec->catdir( @dirs );
	File::Spec->catpath( $volume, $dir, $file );
}
END_NONUNIX
}

1;
